<?php
class Minimax_Partialindex_Model_Resource_Product_Index extends Mage_Core_Model_Mysql4_Abstract
{

    /**
     * Construct
     */
    public function _construct()
    {
        $this->_init('partialindex/product_index','product_id');
    }

    /**
     * List products to reindex
     *
     * @return $this
     */
    public function listProducts(Minimax_Partialindex_Model_Product_Index $object) {
        $connection = $this->_getReadAdapter();

        $fields = array('*');

        $select = $this->_getReadAdapter()->select()
            ->from($this->getMainTable(), $fields);

        $data = $connection->fetchRow($select);

        if($data) $object->setData($data);

        $this->unserializeFields($object);
        $this->_afterLoad($object);

        return $this;
    }

    /**
     * Reindex Rewrite
     *
     * @return Minimax_Partialindex_Model_Resource_Product_Index
     */
    public function reindexAll()
    {
		$this->_reindexUpdatedProducts();
        return $this;
    }
	/**
     * Ids of products which have been created, updated or deleted
     *
     * @return array
     */
    protected function _getProcessedProductIds()
    {
		$maxLimit = (int) Mage::getStoreConfig('minimax_partialindex/index/maxProductReindexed');
		if($maxLimit > 0){
			return Mage::getModel('partialindex/product_index')->getCollection()->distinct(true)->getAllIds($maxLimit);
		}
		return Mage::getModel('partialindex/product_index')->getCollection()->distinct(true)->getAllIds();
	}
	/**
     * Partially reindex newly created and updated products
     *
     * @return Minimax_Partialindex_Model_Resource_Product_Index
     */
    protected function _reindexUpdatedProducts()
    {
        $entityIds = $this->_getProcessedProductIds();
		Mage::log(implode(',',$entityIds),null,'histoIndex.log',true);
		
        /*
         * Generate a fake mass update event that we pass to our indexers.
         */
        $event = Mage::getModel('index/event');
        $event->setNewData(array(
            'reindex_price_product_ids' => &$entityIds, // for product_indexer_price
            'reindex_stock_product_ids' => &$entityIds, // for indexer_stock
            'product_ids'               => &$entityIds, // for category_indexer_product
            'reindex_eav_product_ids'   => &$entityIds  // for product_indexer_eav
        ));

        // Index our product entities.
        Mage::dispatchEvent('partialindex_reindex_products_before_indexer_stock', array('entity_id' => &$entityIds));
        Mage::getResourceSingleton('cataloginventory/indexer_stock')->catalogProductMassAction($event);

        Mage::dispatchEvent('partialindex_reindex_products_before_product_indexer_price', array('entity_id' => &$entityIds));
        Mage::getResourceSingleton('catalog/product_indexer_price')->catalogProductMassAction($event);

        Mage::dispatchEvent('partialindex_reindex_products_before_category_indexer_product', array('entity_id' => &$entityIds));
        Mage::getResourceSingleton('catalog/category_indexer_product')->catalogProductMassAction($event);

        Mage::dispatchEvent('partialindex_reindex_products_before_product_indexer_eav', array('entity_id' => &$entityIds));
        Mage::getResourceSingleton('catalog/product_indexer_eav')->catalogProductMassAction($event);

        Mage::dispatchEvent('partialindex_reindex_products_before_fulltext', array('entity_id' => &$entityIds));
        Mage::getResourceSingleton('catalogsearch/fulltext')->rebuildIndex(null, $entityIds);
        
		Mage::dispatchEvent('partialindex_reindex_products_before_urlrewrite', array('entity_id' => &$entityIds));
		/* @var $urlModel Mage_Catalog_Model_Url */
		$urlModel = Mage::getSingleton('catalog/url');

		$urlModel->clearStoreInvalidRewrites(); // Maybe some products were moved or removed from website
		foreach ($entityIds as $productId) {
			$urlModel->refreshProductRewrite($productId);
		}

        Mage::dispatchEvent('partialindex_reindex_products_before_flat', array('entity_id' => &$entityIds));
        Mage::getSingleton('catalog/product_flat_indexer')->saveProduct($entityIds);

        Mage::dispatchEvent('partialindex_reindex_products_after', array('entity_id' => &$entityIds));

        return $this;
    }
}